@@ -182,7 +182,6 @@ finished_retrieval_cb (WeatherInfo *info, ECalBackendWeather *cbw)
 		obj = icalcomponent_as_ical_string_r (icomp);
 		e_cal_backend_notify_object_created (E_CAL_BACKEND (cbw), obj);
 		g_free (obj);
-		g_object_unref (comp);
 
 		forecasts = weather_info_get_forecast_list (info);
 		if (forecasts) {
@@ -200,7 +199,6 @@ finished_retrieval_cb (WeatherInfo *info, ECalBackendWeather *cbw)
 						obj = icalcomponent_as_ical_string_r (icomp);
 						e_cal_backend_notify_object_created (E_CAL_BACKEND (cbw), obj);
 						g_free (obj);
-						g_object_unref (comp);
 					}
 				}
 			}
@@ -247,13 +245,13 @@ getCategory (WeatherInfo *report)
 		const gchar *icon_name;
 	} categories[] = {
 		{ N_("Weather: Fog"),		"weather-fog" },
-		{ N_("Weather: Cloudy"),	"weather-few-clouds" },
 		{ N_("Weather: Cloudy Night"),	"weather-few-clouds-night" },
+		{ N_("Weather: Cloudy"),	"weather-few-clouds" },
 		{ N_("Weather: Overcast"),	"weather-overcast" },
 		{ N_("Weather: Showers"),	"weather-showers" },
 		{ N_("Weather: Snow"),		"weather-snow" },
-		{ N_("Weather: Sunny"),	"weather-clear" },
 		{ N_("Weather: Clear Night"),	"weather-clear-night" },
+		{ N_("Weather: Sunny"),		"weather-clear" },
 		{ N_("Weather: Thunderstorms"), "weather-storm" },
 		{ NULL,				NULL }
 	};
@@ -265,7 +263,8 @@ getCategory (WeatherInfo *report)
 		return NULL;
 
 	for (i = 0; categories [i].description; i++) {
-		if (g_str_equal (categories [i].icon_name, icon_name))
+		if (!g_ascii_strncasecmp (categories [i].icon_name,
+					      icon_name, strlen(icon_name)))
 			return _(categories [i].description);
 	}
 
@@ -460,7 +459,8 @@ e_cal_backend_weather_get_static_capabilities (ECalBackendSync *backend, EDataCa
 				  CAL_STATIC_CAPABILITY_NO_PROCEDURE_ALARMS  ","
 				  CAL_STATIC_CAPABILITY_NO_TASK_ASSIGNMENT  ","
 				  CAL_STATIC_CAPABILITY_NO_THISANDFUTURE  ","
-				  CAL_STATIC_CAPABILITY_NO_THISANDPRIOR);
+				  CAL_STATIC_CAPABILITY_NO_THISANDPRIOR ","
+				  CAL_STATIC_CAPABILITY_REFRESH_SUPPORTED);
 
 	return GNOME_Evolution_Calendar_Success;
 }
@@ -509,6 +509,29 @@ e_cal_backend_weather_open (ECalBackendSync *backend, EDataCal *cal, gboolean on
 	return GNOME_Evolution_Calendar_Success;
 }
 
+static ECalBackendSyncStatus
+e_cal_backend_weather_refresh (ECalBackendSync *backend, EDataCal *cal)
+{
+	ECalBackendWeather *cbw;
+	ECalBackendWeatherPrivate *priv;
+
+	cbw = E_CAL_BACKEND_WEATHER (backend);
+	priv = cbw->priv;
+
+	if (!priv->opened ||
+	    priv->is_loading)
+		return GNOME_Evolution_Calendar_Success;
+
+	if (priv->reload_timeout_id)
+		g_source_remove (priv->reload_timeout_id);
+	priv->reload_timeout_id = 0;
+
+	/* wait a second, then start reloading */
+	priv->reload_timeout_id = g_timeout_add (1000, (GSourceFunc) reload_cb, cbw);
+
+	return GNOME_Evolution_Calendar_Success;
+}
+
 static ECalBackendSyncStatus
 e_cal_backend_weather_remove (ECalBackendSync *backend, EDataCal *cal)
 {
@@ -787,25 +810,29 @@ e_cal_backend_weather_set_mode (ECalBackend *backend, CalMode mode)
 
 	loaded = e_cal_backend_weather_is_loaded (backend);
 
-	switch (mode) {
-		case CAL_MODE_LOCAL:
-		case CAL_MODE_REMOTE:
-			priv->mode = mode;
-			set_mode = cal_mode_to_corba (mode);
-			if (loaded && priv->reload_timeout_id) {
-				g_source_remove (priv->reload_timeout_id);
-				priv->reload_timeout_id = 0;
-			}
-			break;
-		case CAL_MODE_ANY:
-			priv->mode = mode;
-			set_mode = cal_mode_to_corba (mode);
-			if (loaded && !priv->begin_retrival_id)
-				priv->begin_retrival_id = g_idle_add ((GSourceFunc) begin_retrieval_cb, backend);
-			break;
-		default:
-			set_mode = GNOME_Evolution_Calendar_MODE_ANY;
-			break;
+	if (priv->mode != mode) {
+		switch (mode) {
+			case CAL_MODE_LOCAL:
+			case CAL_MODE_REMOTE:
+				priv->mode = mode;
+				set_mode = cal_mode_to_corba (mode);
+				if (loaded && priv->reload_timeout_id) {
+					g_source_remove (priv->reload_timeout_id);
+					priv->reload_timeout_id = 0;
+				}
+				break;
+			case CAL_MODE_ANY:
+				priv->mode = mode;
+				set_mode = cal_mode_to_corba (mode);
+				if (loaded && !priv->begin_retrival_id)
+					priv->begin_retrival_id = g_idle_add ((GSourceFunc) begin_retrieval_cb, backend);
+				break;
+			default:
+				set_mode = GNOME_Evolution_Calendar_MODE_ANY;
+				break;
+		}
+	} else {
+		set_mode = cal_mode_to_corba (priv->mode);
 	}
 
 	if (loaded) {
@@ -941,6 +968,7 @@ e_cal_backend_weather_class_init (ECalBackendWeatherClass *class)
 	sync_class->get_ldap_attribute_sync = e_cal_backend_weather_get_ldap_attribute;
 	sync_class->get_static_capabilities_sync = e_cal_backend_weather_get_static_capabilities;
 	sync_class->open_sync = e_cal_backend_weather_open;
+	sync_class->refresh_sync = e_cal_backend_weather_refresh;
 	sync_class->remove_sync = e_cal_backend_weather_remove;
 	sync_class->discard_alarm_sync = e_cal_backend_weather_discard_alarm;
 	sync_class->receive_objects_sync = e_cal_backend_weather_receive_objects;
