@@ -19,7 +19,6 @@
  * Boston, MA 02110-1301, USA.
  */
 
-
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -44,7 +43,6 @@
 /* key index debug */
 #define k(x) /*(printf("%s(%d):%s: ",  __FILE__, __LINE__, __PRETTY_FUNCTION__),(x))*/
 
-
 struct _CamelPartitionTablePrivate {
 	pthread_mutex_t lock;	/* for locking partition */
 };
@@ -52,7 +50,6 @@ struct _CamelPartitionTablePrivate {
 #define CAMEL_PARTITION_TABLE_LOCK(kf, lock) (pthread_mutex_lock(&(kf)->priv->lock))
 #define CAMEL_PARTITION_TABLE_UNLOCK(kf, lock) (pthread_mutex_unlock(&(kf)->priv->lock))
 
-
 static void
 camel_partition_table_class_init(CamelPartitionTableClass *klass)
 {
@@ -386,8 +383,6 @@ camel_partition_table_add(CamelPartitionTable *cpi, const gchar *key, camel_key_
 	struct _CamelPartitionKey keys[CAMEL_BLOCK_SIZE/4];
 	gint ret = -1;
 
-#define KEY_SIZE (sizeof(kb->keys)/sizeof(kb->keys[0]))
-
 	hashid = hash_key(key);
 
 	CAMEL_PARTITION_TABLE_LOCK(cpi, lock);
@@ -406,7 +401,7 @@ camel_partition_table_add(CamelPartitionTable *cpi, const gchar *key, camel_key_
 
 	/* TODO: Keep the key array in sorted order, cheaper lookups and split operation */
 
-	if (kb->used < sizeof(kb->keys)/sizeof(kb->keys[0])) {
+	if (kb->used < G_N_ELEMENTS (kb->keys)) {
 		/* Have room, just put it in */
 		kb->keys[kb->used].hashid = hashid;
 		kb->keys[kb->used].keyid = keyid;
@@ -434,8 +429,8 @@ camel_partition_table_add(CamelPartitionTable *cpi, const gchar *key, camel_key_
 			nkb = (CamelPartitionKeyBlock *)&nblock->data;
 		}
 
-		if (pblock && pkb->used < KEY_SIZE) {
-			if (nblock && nkb->used < KEY_SIZE) {
+		if (pblock && pkb->used < G_N_ELEMENTS (kb->keys)) {
+			if (nblock && nkb->used < G_N_ELEMENTS (kb->keys)) {
 				if (pkb->used < nkb->used) {
 					newindex = index+1;
 					newblock = nblock;
@@ -448,7 +443,7 @@ camel_partition_table_add(CamelPartitionTable *cpi, const gchar *key, camel_key_
 				newblock = pblock;
 			}
 		} else {
-			if (nblock && nkb->used < KEY_SIZE) {
+			if (nblock && nkb->used < G_N_ELEMENTS (kb->keys)) {
 				newindex = index+1;
 				newblock = nblock;
 			}
@@ -457,7 +452,7 @@ camel_partition_table_add(CamelPartitionTable *cpi, const gchar *key, camel_key_
 		/* We had no room, need to split across another block */
 		if (newblock == NULL) {
 			/* See if we have room in the partition table for this block or need to split that too */
-			if (ptb->used >= sizeof(ptb->partition)/sizeof(ptb->partition[0])) {
+			if (ptb->used >= G_N_ELEMENTS (ptb->partition)) {
 				/* TODO: Could check next block to see if it'll fit there first */
 				ptnblock = camel_block_file_new_block(cpi->blocks);
 				if (ptnblock == NULL) {
@@ -587,7 +582,6 @@ fail:
 
 /* ********************************************************************** */
 
-
 struct _CamelKeyTablePrivate {
 	pthread_mutex_t lock;	/* for locking key */
 };
@@ -595,7 +589,6 @@ struct _CamelKeyTablePrivate {
 #define CAMEL_KEY_TABLE_LOCK(kf, lock) (pthread_mutex_lock(&(kf)->priv->lock))
 #define CAMEL_KEY_TABLE_UNLOCK(kf, lock) (pthread_mutex_unlock(&(kf)->priv->lock))
 
-
 static void
 camel_key_table_class_init(CamelKeyTableClass *klass)
 {
@@ -650,7 +643,6 @@ camel_key_table_get_type(void)
 	return type;
 }
 
-
 CamelKeyTable *
 camel_key_table_new(CamelBlockFile *bs, camel_block_t root)
 {
@@ -769,7 +761,6 @@ camel_key_table_add(CamelKeyTable *ki, const gchar *key, camel_block_t data, gui
 	}
 #endif
 
-
 	camel_block_file_touch_block(ki->blocks, last);
 	camel_block_file_unref_block(ki->blocks, last);
 
