@@ -1,12 +1,12 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: t; c-basic-offset: 8 -*- */
-/* 
+/*
  * Authors :
  *	Harish Krishnaswamy <kharish@novell.com>
- *   
- * Copyright 2005, Novell, Inc.
  *
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of version 2 of the GNU Lesser General Public 
+ * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of version 2 of the GNU Lesser General Public
  * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -23,17 +23,17 @@
 #ifndef E_GW_RECUR_UTILS_H
 #define E_GW_RECUR_UTILS_H
 
+#include <glib.h>
 
 #define E_GW_ITEM_RECURRENCE_FREQUENCY_DAILY "Daily"
 #define E_GW_ITEM_RECURRENCE_FREQUENCY_WEEKLY "Weekly"
 #define E_GW_ITEM_RECURRENCE_FREQUENCY_MONTHLY "Monthly"
 #define E_GW_ITEM_RECURRENCE_FREQUENCY_YEARLY "Yearly"
 
-
-#define E_GW_ITEM_RECUR_OCCURRENCE_TYPE_FIRST  "First" 
-#define E_GW_ITEM_RECUR_OCCURRENCE_TYPE_SECOND "Second" 
-#define E_GW_ITEM_RECUR_OCCURRENCE_TYPE_THIRD  "Third" 
-#define E_GW_ITEM_RECUR_OCCURRENCE_TYPE_FOURTH "Fourth" 
+#define E_GW_ITEM_RECUR_OCCURRENCE_TYPE_FIRST  "First"
+#define E_GW_ITEM_RECUR_OCCURRENCE_TYPE_SECOND "Second"
+#define E_GW_ITEM_RECUR_OCCURRENCE_TYPE_THIRD  "Third"
+#define E_GW_ITEM_RECUR_OCCURRENCE_TYPE_FOURTH "Fourth"
 #define E_GW_ITEM_RECUR_OCCURRENCE_TYPE_FIFTH  "Fifth"
 #define E_GW_ITEM_RECUR_OCCURRENCE_TYPE_LAST   "Last"
 
@@ -50,23 +50,22 @@
 #define E_GW_ITEM_BY_MONTHDAY_SIZE 32
 #define E_GW_ITEM_BY_YEARDAY_SIZE 367
 #define E_GW_ITEM_BY_MONTH_SIZE 13
+#define E_GW_ITEM_BY_SETPOS_SIZE 367
 
 typedef struct {
-	char *frequency;
-	char *until;
-	int count;
-	int interval;
+	const gchar *frequency;
+	gchar *until;
+	gint count;
+	gint interval;
 	short by_day[E_GW_ITEM_BY_DAY_SIZE];
 	short by_month_day[E_GW_ITEM_BY_MONTHDAY_SIZE];
 	short by_year_day[E_GW_ITEM_BY_YEARDAY_SIZE];
 	short by_month[E_GW_ITEM_BY_MONTH_SIZE];
+	short by_setpos [E_GW_ITEM_BY_SETPOS_SIZE];
 } EGwItemRecurrenceRule;
 
-
 #define E_GW_ITEM_RECUR_END_MARKER  0x7f7f
 
+const gchar *e_gw_recur_get_day_of_week (short day);
 
-
-const char *e_gw_recur_get_day_of_week (short day);
- 
 #endif
